/*
 * Editor.java: the main class
 *
 * Copyright (C) 2000 John Watson, jwatson@tempusmud.com
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class Editor extends JFrame {

        public static String VERSION = "0.19";

        public static String DEFAULT_FLOOR_TEX      = "FlrTexDef";
        public static String DEFAULT_CEILING_TEX    = "CeilTexDef";
        public static String DEFAULT_WALL_TEX       = "WallTexDef";
        public static String DEFAULT_FLOOR_HEIGHT   = "FlrHgtDef";
        public static String DEFAULT_CEILING_HEIGHT = "CeilHgtDef";
        public static String DEFAULT_LIGHT_HEIGHT   = "LgtHgtDef";
        
        private static Properties _props = new Properties();
        private static File _propsFile = null;
        
        public static String getProperty( String name ) { return _props.getProperty( name ); }
        public static void   setProperty( String name, String val ) { _props.setProperty( name, val ); }

        public static boolean saveProperties() {
                if ( ! _propsFile.exists() )
                        return false;

                try {
                        
                        FileOutputStream fos = new FileOutputStream( _propsFile );

                        _props.store( fos, "SDL-DG Editor preferences file" );
                        
                        fos.close();
                        return true;

                }
                catch ( IOException exp ) {
                        JOptionPane.showMessageDialog( null, 
                                                       "An error occured while saving file: " + _propsFile + "\n" +
                                                       exp.getMessage() );
                        return false;
                }
        }

        static {
                String pathname = 
                        System.getProperty( "user.home" ) + File.separator + ".sdl_dg_editor";
                String filename = 
                        pathname + File.separator + "config.props";
                _propsFile = new File( filename );
                
                if ( ! _propsFile.exists() ) {
                        int retval = JOptionPane.showConfirmDialog( null,
                                                                    "I want to create a preferences file called config.props\n" +
                                                                    "in the directory: " + pathname + ".\n" +
                                                                    "Answering no will prevent your settings from saving\n" +
                                                                    "between sessions." );

                        if ( retval == JOptionPane.YES_OPTION ) {
                                File path = new File( pathname );

                                if ( ! path.isDirectory() && ! path.mkdirs() ) {
                                        JOptionPane.showMessageDialog( null, "Unable to create directory: " + pathname );
                                }
                                else {
                                        try {
                                                _propsFile.createNewFile();
                                                
                                        }
                                        catch ( IOException exp ) {
                                                JOptionPane.showMessageDialog( null, 
                                                                               "An error occured while creating file: " + 
                                                                               filename + "\n" +
                                                                               exp.getMessage() );
                                        }
                                }
                        }
                        
                        else {
                                JOptionPane.showMessageDialog( null, "Your settings will not be saved." );
                        }
                }
                else {
                        try {
                        
                                FileInputStream fis = new FileInputStream( _propsFile );

                                _props.load( fis );
                        
                                fis.close();
                        }
                        catch ( IOException exp ) {
                                JOptionPane.showMessageDialog( null, 
                                                               "An error occured while loading file: " + _propsFile + "\n" +
                                                               exp.getMessage() );
                        }
                }
        }

        private JMenuBar _bar = new JMenuBar();
        private JMenu    _viewMenu = new JMenu( "View" );
        private JMenu    _viewMenu_XY = new JMenu( "X-Y" );
        private JMenu    _viewMenu_XZ = new JMenu( "X-Z" );
        private JMenu    _viewMenu_YZ = new JMenu( "Y-Z" );
        
        private JMenu    _editMenu = new JMenu( "Edit" );
        private JMenu    _selectMenu = new JMenu( "Select" );

        private JMenu    _toolsMenu  = new ToolsMenu( this );
        
        private JMenu    _helpMenu   = new JMenu( "Help" );

        private Map      _map;
        private JScrollPane _scroll;
        private Toolbox  _toolbox;
        
        private View2dTop   _view;
        private Controller2dTop _controller;
        
        private ViewControl _vc;
        
        private View2dSide[]   _side_views = new View2dSide[2];
        private Controller2dSide[] _side_controllers = new Controller2dSide[2];
        
        private EditPanel _editPanel;

        private JSplitPane _split;
        
        private WindowListener _windowListener = new WindowAdapter() {
                        public void windowClosing( WindowEvent evt ) {
                                // TODO: something fancy
                                System.out.println( " :: Terminated." );
                                System.exit( 1 );
                        }
                };
        
        public Editor() {
                super( "SDL-DG map editor" );
                
                JMenu file        = new JMenu( "File" );
                JMenuItem newItem = new JMenuItem( "New", 'N' );
                JMenuItem load    = new JMenuItem( "Load", 'L' );
                JMenuItem save    = new JMenuItem( "Save", 'S' );
                JMenuItem export  = new JMenuItem( "Export", 'E' );
                JMenuItem imprt  = new JMenuItem( "Import", 'I' );
                JMenuItem exit    = new JMenuItem( "Exit", 'X' );

                load.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_L, Event.CTRL_MASK ) );
                save.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_S, Event.CTRL_MASK ) );
                export.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_E, Event.ALT_MASK ) );
                exit.setAccelerator( KeyStroke.getKeyStroke( KeyEvent.VK_Q, Event.CTRL_MASK ) );

                newItem.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        Map map = new Map( "Untitled", 
                                                           8192, 
                                                           8192,
                                                           Editor.getProperty( Editor.DEFAULT_FLOOR_TEX ),
                                                           Editor.getProperty( Editor.DEFAULT_CEILING_TEX ),
                                                           Editor.getProperty( Editor.DEFAULT_WALL_TEX ) );
                                        setMap( map );
                                }
                        } );

                load.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        JFileChooser choose = new JFileChooser( System.getProperty( "user.dir" ) );
                                        choose.setSelectedFile( new File( _map.name + "." + Map.JSFMAP_EXT ) );
                                        int choice = choose.showOpenDialog( Editor.this );
                                        
                                        if ( choice == choose.CANCEL_OPTION )
                                                return;
                                        
                                        File f = choose.getSelectedFile();

                                        if ( f != null ) {
                                                Map map = Map.loadMap( f.getAbsolutePath() );
                                                if ( map != null ) {
                                                        setMap( map );
                                                }
                                        }
                                        
                                }
                        } );

                save.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        JFileChooser choose = new JFileChooser( System.getProperty( "user.dir" ) );
                                        choose.setSelectedFile( new File( _map.name + "." + Map.JSFMAP_EXT ) );
                                        int choice = choose.showSaveDialog( Editor.this );

                                        if ( choice == choose.CANCEL_OPTION )
                                                return;

                                        File f = choose.getSelectedFile();

                                        if ( f != null ) {
                                                _map.saveAs( f.getAbsolutePath() );
                                        }
                                }
                        } );

                export.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        JFileChooser choose = new JFileChooser( System.getProperty( "user.dir" ) );
                                        choose.setSelectedFile( new File( _map.name + "." + Map.DGMAP_EXT ) );
                                        choose.showSaveDialog( Editor.this );
                                        File f = choose.getSelectedFile();
                                        
                                        if ( f != null ) {
                                                _map.exportMap( f.getAbsolutePath() );
                                        }
                                }
                        } );
                
                imprt.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        JFileChooser choose = new JFileChooser( System.getProperty( "user.dir" ) );
                                        choose.setSelectedFile( new File( "Unknown" + "." + Map.DGMAP_EXT ) );
                                        choose.showSaveDialog( Editor.this );
                                        File f = choose.getSelectedFile();
                                        
                                        if ( f != null ) {
                                                Map map = Map.importMap( f.getAbsolutePath() );
                                                if ( map != null ) {
                                                        setMap( map );
                                                }
                                        }
                                }
                        } );
                exit.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        System.out.println( " :: Bye" );
                                        System.exit( 0 );
                                }
                        } );

                JMenuItem about = new JMenuItem( "About" );
                
                about.addActionListener( new ActionListener() {
                                public void actionPerformed( ActionEvent evt ) {
                                        showAbout();
                                }
                        } );
                                         
                _bar.add( file );
                _bar.add( _selectMenu );
                _bar.add( _editMenu );
                
                _viewMenu.add( _viewMenu_XY );
                _viewMenu.add( _viewMenu_XZ );
                _viewMenu.add( _viewMenu_YZ );

                _bar.add( _viewMenu );
                _bar.add( _toolsMenu );
                _bar.add( _helpMenu );

                file.add( newItem );
                file.add( load );
                file.add( save );
                file.add( export );
                file.add( imprt );
                file.addSeparator();
                file.add( exit );
                
                _helpMenu.add( about );
                
                setJMenuBar( _bar );

                Container c = getContentPane();

                c.setLayout( new BorderLayout() );

                _map = new Map( "Untitled", 8192, 8192,
                                Editor.getProperty( Editor.DEFAULT_FLOOR_TEX ),
                                Editor.getProperty( Editor.DEFAULT_CEILING_TEX ),
                                Editor.getProperty( Editor.DEFAULT_WALL_TEX ) );
                
                _vc = new ViewControl( _editMenu, _selectMenu );
                
                _view = new View2dTop( _viewMenu_XY );
                _side_views[0] = new View2dSide( _viewMenu_XZ );
                _side_views[1] = new View2dSide( _viewMenu_YZ );
                _side_views[1].setType( _side_views[1].HZ_Y );

                _controller = new Controller2dTop();
                _side_controllers[0] = new Controller2dSide();
                _side_controllers[1] = new Controller2dSide();
                
                _vc.registerViewController( _view, _controller );
                _vc.registerViewController( _side_views[0], _side_controllers[0] );
                _vc.registerViewController( _side_views[1], _side_controllers[1] );


                _vc.setMap( _map );
                
                _side_views[0].setZoom( 0.3f );
                _side_views[1].setZoom( 0.3f );

                _scroll = _view.getScrolledView();
                
                JScrollPane[] _side_scrolls = new JScrollPane[] {
                        _side_views[0].getScrolledView(),
                        _side_views[1].getScrolledView()
                };
                
                _editPanel = new EditPanel( _map );
                
                Dimension mind = new Dimension( 200, 100 );

                _editPanel.setMinimumSize( mind );
                _editPanel.setSize( mind );
                _editPanel.setPreferredSize( mind );

                JPanel viewPanel = new JPanel( new BorderLayout() );
                
                JSplitPane sideSplit = new JSplitPane( JSplitPane.HORIZONTAL_SPLIT,
                                                       _side_scrolls[0],
                                                       _side_scrolls[1] );


                JSplitPane split = new JSplitPane( JSplitPane.VERTICAL_SPLIT,
                                                   _scroll,
                                                   sideSplit );
                                                   
                viewPanel.add( split, "Center" );


                viewPanel.add( _vc.getInfoBar(), "South" );
                
                c.add( viewPanel, "Center" );

                c.add( _editPanel, "East" );
                
                pack();
                setSize( 1024, 768 );
                
                sideSplit.setDividerLocation( 0.5d );
                split.setDividerLocation( 0.6d );
                split.invalidate();
                split.revalidate();

                
                addWindowListener( _windowListener );
        }
        
        void centerViews( boolean sel, boolean multi ) {
                _view.centerView(          sel, multi );
                _side_views[0].centerView( sel, multi );
                _side_views[1].centerView( sel, multi );
        }
        
        void loadMap( String filename ) {
                
                Map map = Map.loadMap( filename );
                
                if ( map != null )
                        setMap( map );
        }

        void setMap( Map map ) {
                _map = map;
                _editPanel.setMap( map );
                _vc.setMap( map );
        }

        Map getMap() { 
                return _map;
        }

        public static void main( String[] args ) {
                Editor ed = new Editor();
                
                if ( args.length > 0 ) {
                        ed.loadMap( args[0] );
                }

                Editor.centerFrame( ed );
                ed.show();
                ed.centerViews( false, false );
        }

        
        private class AboutScreen extends JDialog {
                public AboutScreen( Frame f ) {
                        super( f );
                        AboutScreen.this.setTitle( "About DG Editor" );
                        
                        JTextArea area = new JTextArea();
                        
                        String areaText =
                                "DG Editor\n" +
                                "Version: " + VERSION + "\n" +
                                "...Is a Java-based map editor for the Dungeon-Game.\n" +
                                "It's basically a DOOM map editor... :)\n" +
                                "\n" +
                                "Copyright 2000, John Watson, jwatson@tempusmud.com\n" +
                                "\n" +
                                "See http://tempusmud.com/~jwatson/sdl-dg/editor/\n" +
                                "for more information.";
                        
                        area.setText( areaText );
                        area.setEditable( false );
                        area.setFont( new Font( "SansSerif", Font.PLAIN, 18 ) );
                        JButton ok = new JButton( "Ok" );
                        
                        ok.addActionListener( new ActionListener() {
                                        public void actionPerformed( ActionEvent evt ) {
                                                AboutScreen.this.setVisible( false );
                                                setAboutNull();
                                        }
                                } );

                        Container c = AboutScreen.this.getContentPane();
                        
                        c.setLayout( new GridBagLayout() );

                        GridBagConstraints gbc = new GridBagConstraints();
                        
                        gbc.insets = new Insets( 8, 8, 8, 8 );
                        gbc.gridx = 0;
                        gbc.gridy = 0;
                        gbc.fill = gbc.BOTH;
                        gbc.weightx = gbc.weighty = 1;

                        c.add( area, gbc );
                        
                        ++gbc.gridy;
                        gbc.fill = gbc.NONE;

                        c.add( ok, gbc );
                        
                        this.setLocationRelativeTo( f );
                }
        }

        AboutScreen _aboutScreen = null;

        synchronized private void setAboutNull() {
                _aboutScreen = null;
        }
        
        synchronized public void showAbout() {
                if ( _aboutScreen == null ) {
                        _aboutScreen = new AboutScreen( this );
                }
                _aboutScreen.pack();
                _aboutScreen.show();
        }

        public static void centerFrame( JFrame frame ) {
                Dimension s = Toolkit.getDefaultToolkit().getScreenSize();
                Dimension es = frame.getSize();
                
                frame.setLocation( ( s.width - es.width ) / 2,
                                   ( s.height - es.height ) / 2 );
                
        }
}
